if (typeof(simplemail) == "undefined")
  var simplemail = {a: "mesage.js"};

simplemail.messageTreeView = function() {
  var pub = {
    get rowCount() {
      return simplemail.messageTreeView.messages.length;
    }
  };
  
  pub.messages = [],
  
  pub.getCellText = function(row, column) {
    if (column.id == "image" ||  column.id == "attachmentsCount" || column.id == "read" || column.id == "spam") 
      return null;

    var value = simplemail.messageTreeView.messages[row][column.id];
    switch(column.id) {
      case "date":      return simplemail.SimpleMailDate.toString(value);
      case "size":      return simplemail.size2string(value);
      case "folderId":  return simplemail.overlay.storage.getFolder(value).getName();
      case "color":     return parseInt(value) ? "██" : "";
      default:          return value;
    }
  },
  
  pub.getCellValue = function(row, column) {
    return simplemail.messageTreeView.messages[row].checked;
  },
  
  pub.setCellValue = function(row, column, value) {
    simplemail.messageTreeView.messages[row].checked = value;
    simplemail.messageTreeView.cycleCell(row, column);
  },
  
  pub.isEditable = function(row, column) {
    return column.id == "select";
  },
  
  pub.getImageSrc = function(row, column) {
    if (column.id != "image") return null;

    var message = simplemail.messageTreeView.messages[row];
    var account = simplemail.overlay.storage.getAccount(message.accountId);
    return message.template ? simplemail.SimpleMailFile.getStatusIconURL("notChecked") :
                   !account ? simplemail.SimpleMailFile.getStatusIconURL(message.read ? "noMail" : "mail")
                            : simplemail.SimpleMailFile.getEnvelopeIconURL(account.color);
  },
  
  pub.getCellProperties = function(row, column, props) {
    var message = simplemail.messageTreeView.messages[row];
    var messageProps = new Array();
    if (!message.read) {
      if (props) {
        var atom = simplemail.messageTreeView.atomService.getAtom("unread");
        props.AppendElement(atom);
      } else messageProps.push("unread");
    }
    if (column.id == "attachmentsCount" && simplemail.messageTreeView.messages[row].attachmentsCount) {
      if (props) {
        var atom = simplemail.messageTreeView.atomService.getAtom("attachment");
        props.AppendElement(atom);
      } else messageProps.push("attachment");
    }
    if (column.id == "spam" && message.spam) {
      if (props) {
        var atom = simplemail.messageTreeView.atomService.getAtom("spammail");
        props.AppendElement(atom);
      } else messageProps.push("spammail");
    }
    if (column.id == "read" && !message.read) {
      if (props) {
        var atom = simplemail.messageTreeView.atomService.getAtom("unreadmail");
        props.AppendElement(atom);
      } else messageProps.push("unreadmail");
    }
    if (column.id == "read" && message.read) {
      if (props) {
        var atom = simplemail.messageTreeView.atomService.getAtom("readmail");
        props.AppendElement(atom);
      } else messageProps.push("readmail");
    }
    if (message.color) { // userok: rewrited
      var cval = 'cl' + message.color.toString(16).toUpperCase();
      if (props) {
        var atom = simplemail.messageTreeView.atomService.getAtom(cval);
        props.AppendElement(atom);
      } else 
        messageProps.push(cval);
    }
    if (column.id == "read" && message.read && message.replied) {
      if (props) {
        var atom = simplemail.messageTreeView.atomService.getAtom("readreplied");
        props.AppendElement(atom);
      } else 
        messageProps.push("readreplied");
    }
    if (column.id == "read" && !message.read && message.replied) {
      if (props) {
        var atom = simplemail.messageTreeView.atomService.getAtom("unreadreplied");
        props.AppendElement(atom);
      } else messageProps.push("unreadreplied");;
    }
    if (column.id == "read" && message.read && message.fwd) {
      if (props) {
        var atom = simplemail.messageTreeView.atomService.getAtom("readfwd");
        props.AppendElement(atom);
      } else messageProps.push("readfwd");
    }
    if (column.id == "read" && !message.read && message.fwd) {
      if (props) {
        var atom = simplemail.messageTreeView.atomService.getAtom("unreadfwd");
        props.AppendElement(atom);
      } else messageProps.push("unreadfwd");
    }
    return messageProps.join(" ");
  },

  // userok: rewrited
  pub.getRowProperties = function(row, props) {
    var message = simplemail.messageTreeView.messages[row];
    if (message.color) {
      var cval = 'cl' + message.color.toString(16).toUpperCase();
      if (props) {
        var atom = simplemail.messageTreeView.atomService.getAtom(cval);
        props.AppendElement(atom);
      } else 
        return cval;
    }
  },
  
  pub.getString = function(statement, id) {
    var text = "text: ";
    var value = statement.getString(id);
    return value && value.substr(0, text.length) == text ? value.substr(text.length) : value;
  },
  
  pub.cycleHeader = function(column) {
    column = simplemail.domGet(column.id);

    if (column.id == "select") {
      var checked = true;
      for (var i = 0; i < simplemail.messageTreeView.messages.length; i++)
        checked &= simplemail.messageTreeView.messages[i].checked == "true";

      for (var i = 0; i < simplemail.messageTreeView.messages.length; i++)
        simplemail.messageTreeView.messages[i].checked = !checked ? "true" : "false";

      return;
    }

    var sortedColumn = simplemail.messageTreeView.getSortedColumn();
    if (sortedColumn && sortedColumn != column)
      sortedColumn.removeAttribute("sortDirection");
      
    var direction = column.getAttribute("sortDirection");
    column.setAttribute("sortDirection", simplemail.messageTreeView.nextDirection[direction]);
    simplemail.messageTreeView.displayFolder();
  },
  
  // userok: rewrited
  pub.onClick = function(event) {
    //console.error("SM.messageTreeView_onClick", event);//userok_log
  },
  
  // userok: rewrited
  pub.onSelect = function(event) {
    //console.error("SM.messageTreeView_onSelect", event, this);//userok_log
    simplemail.messageTreeView.displayMessageHtml();
  },  

  // userok: rewrited
  pub.action = function() { 
    if (simplemail.messageTreeView.isPartialSelected())
      simplemail.messageTreeView.downloadMessageBody();
    else if (simplemail.SimpleMailManageFolder.getCurrentFolder().id == simplemail.Folder.DRAFTS) {
      var message = simplemail.overlay.storage.getMessage(simplemail.messageTreeView.messages[messageTree.currentIndex].id);
      simplemail.overlay.openNewMessage(message);
    } else if (simplemail.SimpleMailPrefs.getBool('messagesInWindow'))
      simplemail.messageTreeView.openNewTabOrWindow(true);
    else if (simplemail.SimpleMailPrefs.getBool('messagesInTab'))
      simplemail.messageTreeView.openNewTabOrWindow(false);
    else
      simplemail.messageTreeView.toggleRead();
  },

  pub.onSpamClicked = function(event) {
    if (event.type == "dblclick") 
      simplemail.overlay.stop();
    else 
      toggletoSpam();
  },
  
  pub.forEachMessagesSelected = function(onItem) {
    if (!simplemail.messageTreeView.forEachChecked(onItem))
      simplemail.messageTreeView.forEachSelected(onItem);
  },
  
  pub.delete = function (event) {
    if (event)
      simplemail.messageTreeView.deleteMail(true);
    else
      simplemail.messageTreeView.deleteMail();
  },

  // userok
  pub.openNewTabOrWindow = function(bNewWindow) {
    if (simplemail.SimpleMailManageFolder.getCurrentFolder().id == simplemail.Folder.DRAFTS)
      return;   
    
    var messageTree = simplemail.domGet("message-tree");
    var aMessage    = simplemail.messageTreeView.messages[messageTree.currentIndex];  
    var url         = "chrome://simplemail/content/main/message.xul?" + aMessage.id;

    // Window exist ?
    if (bNewWindow) {
      for (var i = 0; i < simplemail.overlay.openedWindows.length; i++) {
        if (simplemail.overlay.openedWindows[i].name == aMessage.id) {
          simplemail.overlay.openedWindows[i].focus();
          return;
        }
      }
    }
    
    if (bNewWindow) {
      var msgwnd = openDialog(url, aMessage.id, "chrome,dialog=no,height=400,width=700,resizable,centerscreen");
      simplemail.overlay.openedWindows.push(msgwnd);
    } else {
      var gBrowser = simplemail.getBrowser();
      if (content.location != "about:blank")
        gBrowser.selectedTab = gBrowser.addTab(url)
      else
        content.location.assign(url);      
    }
  }
  
  return pub;
}();

simplemail.DefaultTreeView.extend(simplemail.messageTreeView);

// userok: rewrited
simplemail.messageTreeView.displayMessageHtml = function() {
  var body = simplemail.domGet("body");
  body.contentWindow.scroll(0, 0);

  var message = (messageTree.currentIndex != -1) ? simplemail.messageTreeView.messages[messageTree.currentIndex] : {};

  simplemail.messageTreeView.displayHeaders(message);

  var html = message.partial
               ? simplemail.Bundle.getString("messageNotDownloaded")
               : message.id ? simplemail.overlay.storage.getMessage(message.id).html : " ";

  var search = simplemail.domGet("search").value;
  if (search) 
    html = simplemail.SimpleMailText.highlight(search, html);

  var originalHTML = html;
  
  if (html !== " ") {
    var source = simplemail.SimpleMailFile.getAttachmentsDirectory(message.attachmentsDir);
    source.append(simplemail.SimpleMailFile.MESSAGE_SOURCE);
    source = simplemail.SimpleMailFile.readFile(source);
  }

  if (simplemail.SimpleMailPrefs.getBool("plaintextOnReceive") && html !== " ") {
    var partAttachs = simplemail.messageTreeView.displayAttachments(html, message);

    var attachments = [];
    simplemail.SimpleMailMessageEncoder.cidNames = [];
    simplemail.SimpleMailMessageEncoder.ContentId = [];
    simplemail.SimpleMailMessageEncoder.ContentDisposition = [];
    simplemail.SimpleMailMessageEncoder.ContentType = [];
    var tmpMessage = simplemail.SimpleMailMessageEncoder.parse(source, attachments, simplemail.SimpleMailMessageEncoder.cidNames, simplemail.SimpleMailMessageEncoder.ContentId, simplemail.SimpleMailMessageEncoder.ContentDisposition, simplemail.SimpleMailMessageEncoder.ContentType);

    if (!source.match(/name\=/gim) && !source.match(/filename\=/gim) && !source.match(/Content\-ID\:/gim) || partAttachs == null) {  // No Attachments
      var html = simplemail.parseHTML(document, tmpMessage.html, false);
      html = simplemail.messageTreeView.plaintextMessage(html);
      html = html.replace(/(\r\n).*?/g,"<br>"); 
    } else {
      if (partAttachs != null)
        partAttachs = partAttachs.join("");

      if (source.match(/text\/plain/gim)) { // Plaintext part in the message
        var html = tmpMessage.html + "<br>" + partAttachs;
        var html = simplemail.parseHTML(document, html, true);
        html = simplemail.messageTreeView.plaintextMessage(html);
      } else {
        // Remove Inline images
        var getImages = tmpMessage.html.toString().match(/<img[^>]*>(.*?)>*/gim); 
        if (getImages != null){
          for (var i in getImages){
            tmpMessage.html = tmpMessage.html.replace(getImages[i], "");
          }
        }
        var html = simplemail.parseHTML(document, tmpMessage.html, true);
        html = simplemail.messageTreeView.plaintextMessage(html);
        html = html.replace(/(\r\n).*?/g,"<br>");   
        html = html  + "<br>" + partAttachs;
      }
    }

    if (!(simplemail.overlay.storage.getMessage(message.id).html).match(/\<a\sclass\=\'attachment\'/gim)) {
      message.html = originalHTML;
      simplemail.overlay.storage.updateMessage(0, message); // userok: async
    }
  } else {
    html = simplemail.messageTreeView.displayAttachments(html, message);
  }

  html = simplemail.SimpleMailText.imageToLink(html);
  
  // Change : remove : i don't remember what does this does !!
  /*if (html !== " " && source.toString().match(/Content\-Type\: text\/plain/gim) && simplemail.SimpleMailPrefs.getBool("plaintextOnReceive")) {
    html = simplemail.SimpleMailText.linkToLink(html);
  }*/
  
  body.contentDocument.title = message.subject;

  if (simplemail.SimpleMailPrefs.getBool("plaintextOnReceive")) {
    var getHtmlImages = html.toString().match(/<img[^>]*>(.*?)>*/gim);
    if (getHtmlImages != null) {
      for (var k in getHtmlImages) { 
        if (getHtmlImages[k].match(/http/gim))
          var html = html.replace(getHtmlImages[k], simplemail.Bundle.getString("removedimages"));
      }
    }
    html = simplemail.SimpleMailText.linkToText(html);
  }

  // userok
  var fragment = simplemail.parseHTML(body.contentDocument, html, true, simplemail.SimpleMailPrefs.getBool('blockmedia'));
  body.contentDocument.body.innerHTML = "";
  body.contentDocument.body.appendChild(fragment);
  if (message.returnReceiptTo && message.uid && !message.partial)
    simplemail.messageTreeView.sendReturnReceipt(message);

  clearTimeout(simplemail.messageTreeView.markReadTimeout);
  
  if (!message.read && !message.partial)
    simplemail.messageTreeView.startMarkReadTimeout(message);
}

simplemail.messageTreeView.plaintextMessage = function (html) {
  var body = simplemail.domGet("body");
  body.contentDocument.body.innerHTML = "";
  var fonts = body.contentDocument.createElement("font");
  fonts.setAttribute("face", "Consolas");
  var range = document.createRange();
  fonts.appendChild(html);
  var sp2 = body.contentDocument.body.getElementsByTagName("body");
  body.contentDocument.body.insertBefore(fonts, sp2.nextSibling);
  html = body.contentDocument.body.innerHTML
  return html = simplemail.sanitizeHtml(html, true);     
}

simplemail.messageTreeView.attachmentHeader = function () {
  var body = simplemail.domGet("body");
  body.contentDocument.body.innerHTML = "";
  var attachDiv = body.contentDocument.createElement("div");
  attachDiv.setAttribute("style", "border-radius: 5px; background-color: gray;");
  attachDiv.setAttribute("class", "line");
  var attachString = body.contentDocument.createTextNode(simplemail.Bundle.getString("attachments") + " : ");
  var brTag1 = body.contentDocument.createElement("br");
  var brTag2 = body.contentDocument.createElement("br");
  var brTag3 = body.contentDocument.createElement("br");
  var brTag4 = body.contentDocument.createElement("br");
  attachDiv.appendChild(brTag1);
  attachDiv.appendChild(attachString);
  attachDiv.appendChild(brTag2);
  attachDiv.appendChild(brTag3);
  body.contentDocument.body.appendChild(brTag4);
  body.contentDocument.body.appendChild(attachDiv);
  return body.contentDocument.body.innerHTML;
}

simplemail.messageTreeView.displayAttachments = function(html, message) {
  if (html == " ")
    return " ";

  var messageHtml = html;
  var source = simplemail.SimpleMailFile.getAttachmentsDirectory(message.attachmentsDir);
  source.append(simplemail.SimpleMailFile.MESSAGE_SOURCE);
  source = simplemail.SimpleMailFile.readFile(source);
  if (!source.match(/name\=/gim) && !source.match(/filename\=/gim) && !source.match(/Content\-ID\:/gim))
    return html;

  var htmlTrimed = html.replace(/[\t\n\r ]+/g, " ");
  var getHtml = htmlTrimed.match(/<html[^>]*>(.*?)<\/html>/gim);
  if (getHtml == null) {
    htmlTrimed = "<html>" + htmlTrimed + "</html>";
    getHtml = htmlTrimed.match(/<html[^>]*>(.*?)<\/html>/gim);
  }

  // Get attachments in the body
  var getAttachs1 = [];
  getAttachs1 = getHtml.toString().match(/<a\sclass\=\'attachment\'[^>]*>(.*?)<\/a>*/gim);
  if (getAttachs1 != null) {
    for (var i in getAttachs1) {
      htmlTrimed = htmlTrimed.replace(getAttachs1[i], ""); // Delete attachs in the body
      var htmlParsed = htmlTrimed + getAttachs1[i];  // Add attachs after body
    }
  }
  
  // Get attachments not in the body
  if (getAttachs1 == null) {
    var getAttachs2 = []; 
    getAttachs2 = htmlTrimed.toString().match(/<a\sclass\=\'attachment\'[^>]*>(.*?)<\/a>*/gim);
    if (getAttachs2 != null)
      var htmlParsed = htmlTrimed;   // Add attachs after body
  }  

  if (simplemail.SimpleMailPrefs.getBool("plaintextOnReceive")){
    var getAttachs3 = [];
    getAttachs3 = getHtml.toString().match(/<img[^>]*>(.*?)>*/gim);
    if (getAttachs3 != null) {
      for (var j in getAttachs3) {
        htmlTrimed = htmlTrimed.replace(getAttachs3[j], ""); // Delete attachs in the body
        htmlTrimed = htmlTrimed+getAttachs3[j]; // Add images attachs after body
      }
      if (getAttachs1 != null) {
        for (var i in getAttachs1)
          htmlTrimed = htmlTrimed + getAttachs1[i];
      }
      var htmlParsed = htmlTrimed;
    }
  }

  if (getAttachs1 != null)
    var getAttachs = getAttachs1.concat(getAttachs2, getAttachs3);
  else if (getAttachs2 != null)
    var getAttachs =  getAttachs2.concat(getAttachs3);
  else if (getAttachs3 != null)
    var getAttachs = getAttachs3;

  if (getAttachs == null || typeof(getAttachs) == 'undefined') {
    var getAttachs = null;
    var htmlParsed = htmlTrimed;
  } 

  html = htmlParsed;
  var attachments = [];
  simplemail.SimpleMailMessageEncoder.cidNames = [];
  simplemail.SimpleMailMessageEncoder.ContentId = [];
  simplemail.SimpleMailMessageEncoder.ContentDisposition = [];
  simplemail.SimpleMailMessageEncoder.ContentType = [];
  
  var newMessage = simplemail.SimpleMailMessageEncoder.parse(source, attachments, simplemail.SimpleMailMessageEncoder.cidNames, simplemail.SimpleMailMessageEncoder.ContentId, simplemail.SimpleMailMessageEncoder.ContentDisposition, simplemail.SimpleMailMessageEncoder.ContentType);

  for (var j in simplemail.SimpleMailMessageEncoder.ContentDisposition) {
    var disposition = simplemail.SimpleMailMessageEncoder.ContentDisposition[j];
    var regexp = /(?:[^;\s]*)/; 
    var Disposition =  regexp.exec(disposition);
    if (((Disposition == "attachment" || simplemail.SimpleMailPrefs.getBool("plaintextOnReceive")) && !html.toString().match(/\<div\sclass\=\"line\"/gim)) && simplemail.SimpleMailPrefs.getBool("newAttachs")) {
      html += simplemail.messageTreeView.attachmentHeader();
      html = html.replace(/(<\/div>)$/g, "");
    }

    var getName = /name\= *['"]?([^;\r"]*)/i;
    var type = simplemail.SimpleMailMessageEncoder.ContentType[j];
    var filename = getName.exec(type);
    if (filename !== null) {
      filename[0] = filename[0].replace(/[\'\[\]\(\)]/gi, "_")
      filename[1] = filename[1].replace(/[\'\[\]\(\)]/gi, "_")
      var file = simplemail.SimpleMailFile.getAttachmentsDirectory(message.attachmentsDir);
      file.append(filename[1]);
      if (!file.exists()) {
        var getName = /name\= *['"]?([^;\r'"]*)/i;   // To match old attachments files
        var filename = getName.exec(type);
        filename[0] = filename[0].replace(/[\'\[\]\(\)]/gi, "_")
        filename[1] = filename[1].replace(/[\'\[\]\(\)]/gi, "_")
      }
    } else {
      var getFileName = /filename\= *['"]?([^;\r"]*)/i;
      var filename = getFileName.exec(disposition);
      if (filename !== null) {
        filename[0] = filename[0].replace(/[\'\[\]\(\)]/gi, "_")
        filename[1] = filename[1].replace(/[\'\[\]\(\)]/gi, "_")
        var file = simplemail.SimpleMailFile.getAttachmentsDirectory(message.attachmentsDir);
        file.append(filename[1]);
        if (!file.exists()) {
          var getFileName = /filename\= *['"]?([^;\r'"]*)/i;  // To match old attachments files
          var filename = getFileName.exec(disposition);
          filename[0] = filename[0].replace(/[\'\[\]\(\)]/gi, "_")
          filename[1] = filename[1].replace(/[\'\[\]\(\)]/gi, "_")
        }
      } else {
        for (var k in simplemail.SimpleMailMessageEncoder.ContentId)
          var cid = simplemail.SimpleMailMessageEncoder.ContentId[k];

        var filename = simplemail.SimpleMailMessageEncoder.cidNames[cid];
        if (typeof(filename) !== 'undefined') {
          var file = simplemail.SimpleMailFile.getAttachmentsDirectory(message.attachmentsDir);
          file.append(filename);
          if (!file.exists())
            filename = filename.toString().replace(/[\'\[\]\(\)]/gi, "_")
        }
      }
    }
    
    // Create attachment file if not exist
    var file = simplemail.SimpleMailFile.getAttachmentsDirectory(message.attachmentsDir);
    file.append(filename[1]);
    if (!file.exists()) {
      for (var name in attachments) {
        name = name.toString().replace(/[\'\[\]\(\)] */gi, "_");
        simplemail.SimpleMailFile.saveAttachment(message.attachmentsDir, name, attachments[name]);
      }
    }
    
    var regexp = new RegExp ('alt="' + filename[1], "gim");
    var result = regexp.exec(newMessage.html);
    if (result == null & Disposition == "inline")
      result = true;

    var myArray = html.match(/<html[^>]*>(.*?)<\/html>/gim);  // Get html part
    if (myArray != null) {
      if ((message.folderId == 3 || message.folderId == 2) & (Disposition == "attachment" || result == true)) { // Sent and drafts folders
        if (!html.match(/\<div\sclass\=\"line\"/gim) && simplemail.SimpleMailPrefs.getBool("newAttachs")) {
          html += simplemail.messageTreeView.attachmentHeader();
          html = html.replace(/(<\/div>)$/g,"");
        }

        var newUrl = simplemail.SimpleMailFile.getAttachmentURL(message.attachmentsDir, filename[1]);
        if (!myArray.toString().match(newUrl)){  // Check if inline file exist
          html += simplemail.SimpleMailMessageEncoder.getAttachmentHtml(newUrl);
          var draftDisposition = true;
          messageHtml += simplemail.SimpleMailMessageEncoder.getAttachmentHtml(newUrl, draftDisposition);
          var myArray = html.match(/<html[^>]*>(.*?)<\/html>/gim);  // Get html part
        }
      }
      try {
        var myArray = decodeURIComponent(myArray);
      } catch(e) {
        myArray = myArray;
      }

      if (typeof(filename) == 'undefined')
        var filename = getAttachs.toString().match(/>([^<]*)</);

      if (typeof(filename) !== 'undefined')
        var checkInline = myArray.toString().match(filename[1]);  // Check if inline file exist
    }

    var newAttachment = "";

    if (Disposition == "attachment" || checkInline == null) {
      if (!html.match(/\<div\sclass\=\"line\"/gim) && simplemail.SimpleMailPrefs.getBool("newAttachs")) {
        html += simplemail.messageTreeView.attachmentHeader();
        html = html.replace(/(<\/div>)$/g,"");
      }

      var attachmentsURL = simplemail.SimpleMailFile.getAttachmentURL(message.attachmentsDir, filename[1]);
      // Remove attachement
      html = simplemail.SimpleMailText.replaceURLs(html,
      function(text, url) {
        if (attachmentsURL == url)
          return " ";
        else
          return text;
      });

      var background = simplemail.SimpleMailMessageEncoder.getIconForAttachment(filename[1]);
      var attachmentSize = simplemail.SimpleMailFile.readURL(attachmentsURL).length;
      attachmentSize = simplemail.size2string(attachmentSize);
      var body = simplemail.domGet("body");
      body.contentDocument.body.innerHTML = "";
      var attachClass = body.contentDocument.createElement("a");
      attachClass.setAttribute("style", "background: url(" + background + ")center left no-repeat;");
      attachClass.setAttribute("href", attachmentsURL);
      attachClass.setAttribute("class", "newAttachment");      
      var attachName = body.contentDocument.createTextNode(filename[1]);
      var attachSize = body.contentDocument.createTextNode(" (" + attachmentSize + ")");
      var brTag1 = body.contentDocument.createElement("br");
      var brTag2 = body.contentDocument.createElement("br");
      var brTag3 = body.contentDocument.createElement("br");
      attachClass.appendChild(attachName);
      attachClass.appendChild(attachSize);
      attachClass.appendChild(brTag2);
      attachClass.appendChild(brTag3);
      body.contentDocument.body.appendChild(attachClass);      
      html += body.contentDocument.body.innerHTML;
    }
  } 
  
  if (!(simplemail.overlay.storage.getMessage(message.id).html).match(/\<a\sclass\=\'attachment\'/gim) && message.folderId == 3) {
    message.html = messageHtml;
    simplemail.overlay.storage.updateMessage(0, message); // userok: async
  }

  if (html.match(/\<div\sclass\=\"line\"/gim))
    html = html+"</div>";

  if (simplemail.SimpleMailPrefs.getBool("newAttachs"))
    var partAttachs = html.toString().match(/\<div\sclass\=\"line\"[^>]*>(.*?)<\/div>*/gim);
  else
    var partAttachs = html.toString().match(/\<a\sclass\=\"newAttachment\"[^>]*>(.*?)<\/a>*/gim);

  if (simplemail.SimpleMailPrefs.getBool("plaintextOnReceive"))
    return partAttachs;
  else
    return html;
}

simplemail.messageTreeView.displayHeaders = function(message) {
  simplemail.domGet("headers-from").value = message.from || "";
  simplemail.domGet("headers-to").value = message.to || "";
  simplemail.domGet("headers-cc").value = message.cc || "";
  simplemail.domGet("headers-bcc").value = message.bcc || "";
  simplemail.domGet("headers-subject").value = message.subject || "";
  simplemail.domGet("headers-date").value = simplemail.SimpleMailDate.toString(message.date);
  if (simplemail.SimpleMailPrefs.getBool("plaintextOnReceive") & !simplemail.SimpleMailPrefs.getBool("warningPlainText")) {
    simplemail.domGet("headers-plaintextonreceive").collapsed = false;
  }
  else {
    simplemail.domGet("headers-plaintextonreceive").collapsed = true;
  }
}

simplemail.messageTreeView.sendReturnReceipt = function(message) {
  function clearReturnReceiptTo() {
    message.returnReceiptTo = null;
    simplemail.overlay.storage.saveMessageState(message);
  }
  if (simplemail.SimpleMailPrefs.getString("sendreceiptresp") == "asksend") {
    var prompts = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
                  .getService(Components.interfaces.nsIPromptService);
    var button = prompts.confirmEx(window, "Simple Mail",
                                   simplemail.Bundle.getString("sendReturnReceipt"),
                                   prompts.BUTTON_POS_0 * prompts.BUTTON_TITLE_YES +
                                   prompts.BUTTON_POS_1 * prompts.BUTTON_TITLE_NO +
                                   prompts.BUTTON_POS_2 * prompts.BUTTON_TITLE_IS_STRING,
                                   null, null, simplemail.Bundle.getString("later"), null, {});
    if (button == 0) { // "Yes"
      var account = simplemail.overlay.storage.getAccount(message.accountId);
      var receipt = message.createReturnReceipt(account.toString(), message);
      simplemail.SimpleMailPrefs.setString('originalTo', receipt.to);
      simplemail.SimpleMailPrefs.setString('originalCc', receipt.cc);
      simplemail.SimpleMailPrefs.setString('originalBcc', receipt.bcc);
      simplemail.overlay.autosendMail(receipt, clearReturnReceiptTo);
    }
    else if (button == 1) { // "No"
      clearReturnReceiptTo();
    }
  }
  else if (simplemail.SimpleMailPrefs.getString("sendreceiptresp") == "alwayssend") {
    var account = simplemail.overlay.storage.getAccount(message.accountId);
    var receipt = message.createReturnReceipt(account.toString(), message);
    simplemail.SimpleMailPrefs.setString('originalTo', receipt.to);
    simplemail.SimpleMailPrefs.setString('originalCc', receipt.cc);
    simplemail.SimpleMailPrefs.setString('originalBcc', receipt.bcc);
    simplemail.overlay.autosendMail(receipt, clearReturnReceiptTo);
  }
  else if (simplemail.SimpleMailPrefs.getString("sendreceiptresp") == "neversend") {
    clearReturnReceiptTo();
  }
}

simplemail.messageTreeView.original = new Array();

simplemail.messageTreeView.autoForward = function(message) {
  function setautoForward() {
    message.fwd = true;
    simplemail.overlay.storage.saveMessageState(message);
  }

  var account = simplemail.overlay.storage.getAccount(message.accountId);
  
  // If exist, delete coma in name    
  message.to  = simplemail.SimpleMailNewMessage.cleanAddresses(message.to);  
  message.cc  = simplemail.SimpleMailNewMessage.cleanAddresses(message.cc); 
  message.bcc = simplemail.SimpleMailNewMessage.cleanAddresses(message.bcc);
  
  var messageForward = simplemail.overlay.storage.getMessage(message.id).createForward();
  var forwardParam = simplemail.SimpleMailPrefs.getString('forwardParam');
  messageForward.to = forwardParam;
  messageForward.from = message.from;
  simplemail.overlay.autosendMail(messageForward, setautoForward);
}

simplemail.messageTreeView.markReadTimeout = null;

simplemail.messageTreeView.startMarkReadTimeout = function(message) {
  var time = simplemail.SimpleMailPrefs.getInt("markReadTimeout");
  if (time == 0 && simplemail.messageTreeView.messages[messageTree.currentIndex] &&
        simplemail.messageTreeView.messages[messageTree.currentIndex].id == message.id)
  {
    simplemail.messageTreeView.markRead(messageTree.currentIndex, true);
  };
   
  if (!time) return;

  simplemail.messageTreeView.markReadTimeout = setTimeout(function() {
    if (simplemail.messageTreeView.messages[messageTree.currentIndex] &&
        simplemail.messageTreeView.messages[messageTree.currentIndex].id == message.id)
    {
      simplemail.messageTreeView.markRead(messageTree.currentIndex, true);
    }
  }, time * 1000);
}

// Mark read individual message
simplemail.messageTreeView.markRead = function(index, read) {
  var message = simplemail.messageTreeView.messages[index];
  message.read = read;
  simplemail.overlay.storage.saveMessageState(message);
  var account  = simplemail.overlay.storage.getAccount(message.accountId);
  var flag     = message.read == 1 ? "Seen" : "UnSeen"
  var folder   = simplemail.SimpleMailManageFolder.getCurrentFolder().name;
//userok: turn off test  var messages = [];
//userok: turn off test  messages.push(simplemail.messageTreeView.messages[index]);
//userok: turn off test  simplemail.overlay.SynchroMail(account, messages, folder, flag);
  simplemail.messageTreeView.treebox.invalidateRow(index);
  simplemail.SimpleMailManageFolder.updateFolderCounts();
}

simplemail.messageTreeView.displayMessageSource = function() {
  var attachmentsDir = simplemail.messageTreeView.messages[messageTree.currentIndex].attachmentsDir;
  var url = simplemail.SimpleMailFile.getAttachmentURL(attachmentsDir, simplemail.SimpleMailFile.MESSAGE_SOURCE);
  var gBrowser = simplemail.getBrowser();
  gBrowser.selectedTab = gBrowser.addTab(url);
}

// If there's an unread message selected, mark all selected simplemail.messageTreeView.messages read.
// Otherwise mark all selected simplemail.messageTreeView.messages unread.
simplemail.messageTreeView.toggleRead = function() { // userok: shit code again?
  var read = false;
  simplemail.messageTreeView.forEachMessagesSelected(function(index) {
    read |= !simplemail.messageTreeView.messages[index].read;
  });
  simplemail.messageTreeView.forEachMessagesSelected(function(index) {
    simplemail.messageTreeView.markRead(index, read);
  });
}

simplemail.messageTreeView.deleteMail = function(skipTrash) {
  if (messageTree.view.selection.getRangeCount() == 0) 
    return;

  if (!skipTrash && simplemail.SimpleMailManageFolder.getCurrentFolder().id != simplemail.Folder.TRASH && simplemail.SimpleMailPrefs.getBool("moveToTrash")) 
    return simplemail.messageTreeView.moveToFolder(simplemail.Folder.TRASH); // userok: async

  var msgs = [];
  simplemail.messageTreeView.forEachMessagesSelected(function(index) {
    msgs.push(simplemail.messageTreeView.messages[index]);
  });

  simplemail.overlay.askDeleteMail(msgs, simplemail.Bundle.getString("deleteMessages"), window);
}

simplemail.messageTreeView.onMessageAdded = function(message) {
  var folder = simplemail.SimpleMailManageFolder.getCurrentFolder();
  if (folder && folder.id == message.folderId) {
    simplemail.messageTreeView.messages.splice(0, 0, message);
    simplemail.messageTreeView.treebox.rowCountChanged(0, 1);
  }
  simplemail.SimpleMailManageFolder.updateFolderCounts();
}

simplemail.messageTreeView.onMessageUpdated = function(message) {
  for (var i = 0; i < simplemail.messageTreeView.messages.length; i++) {
    if (simplemail.messageTreeView.messages[i].id == message.id) {
      simplemail.messageTreeView.messages[i] = message;
      simplemail.messageTreeView.treebox.invalidateRow(i);
      if (i == messageTree.currentIndex) simplemail.messageTreeView.displayMessageHtml();
      break;
    }
  }
}

simplemail.messageTreeView.openNew = function(toAddress) {
  var message = new simplemail.Message();
  message.to  = toAddress;
  simplemail.overlay.openNewMessage(message);
}

// userok: rewrited
simplemail.messageTreeView.openReply = function(all) {
  //console.error("SM.messageTreeView_openReply", all, simplemail.overlay);//userok_log
  if (messageTree.currentIndex == -1)
    return;
  
  var LMessage = simplemail.messageTreeView.messages[messageTree.currentIndex];

  simplemail.SimpleMailPrefs.setString('originalId', LMessage.id);
  var reply = simplemail.overlay.storage.getMessage(LMessage.id).createReply(all);
  simplemail.overlay.openNewMessage(reply);
}

// userok: rewrited
simplemail.messageTreeView.openForward = function(toAddress) {
  if (messageTree.currentIndex == -1)
    return;
  
  var LMessage = simplemail.messageTreeView.messages[messageTree.currentIndex];

  simplemail.SimpleMailPrefs.setString('originalId', LMessage.id);
  var forward = simplemail.overlay.storage.getMessage(LMessage.id).createForward();
  forward.to  = toAddress;
  simplemail.overlay.openNewMessage(forward);
}

// userok: rewrited
simplemail.messageTreeView.doprintMessage = function(aMessage) {
  var msg         = simplemail.overlay.storage.getMessage(aMessage.id);
  var messageHTML = msg.toHtml();
  var fragment    = simplemail.parseHTML(document, messageHTML, true);
  var printer     = simplemail.domGet("printer");
  printer.contentDocument.body.appendChild(fragment);
  printer.contentWindow.print();
  simplemail.clearElement(printer.contentDocument.body);
}

// userok: rewrited
simplemail.messageTreeView.printMessage = function() {
  if (messageTree.currentIndex != -1)
    simplemail.messageTreeView.doprintMessage(simplemail.messageTreeView.messages[messageTree.currentIndex]);
}

// userok: rewrited
simplemail.messageTreeView.dosaveMessageAs = function(aMessage) {  
  var fp = Components.classes["@mozilla.org/filepicker;1"].createInstance(Components.interfaces.nsIFilePicker);
  fp.init(window, null, fp.modeSave);
  fp.appendFilters(fp.filterHTML);
  fp.appendFilter("*.eml", "*.eml");
  fp.defaultString = aMessage.subject;

  var result = fp.show();
  if (result == fp.returnCancel) 
    return;

  var file = fp.file.clone();
  if (fp.filterIndex == 0) {
    if (!file.leafName.match(/\.html?$/))
      file.leafName += ".html";
    simplemail.messageTreeView.saveAsHtml(file, aMessage);
  } else {
    if (!file.leafName.match(/\.eml$/))
      file.leafName += ".eml";
    simplemail.messageTreeView.saveAsEmail(file, aMessage);
  }
}

// userok: rewrited
simplemail.messageTreeView.saveMessageAs = function() {
  if (messageTree.currentIndex != -1) 
    simplemail.messageTreeView.dosaveMessageAs(simplemail.messageTreeView.messages[messageTree.currentIndex]);
}

simplemail.messageTreeView.selectFirstMessage = function() {
  if (simplemail.messageTreeView.rowCount > 0) {
    messageTree.view.selection.select(0);
    simplemail.messageTreeView.treebox.scrollToRow(0);
  }
}

simplemail.messageTreeView.displayFolder = function() {
  var folder = simplemail.SimpleMailManageFolder.getCurrentFolder();

  var isSentOrDraft = folder.id == simplemail.Folder.SENT || folder.id == simplemail.Folder.DRAFTS;
  var bothVisible = simplemail.domGet("from").hidden == simplemail.domGet("to").hidden;
  simplemail.domGet("from").setAttribute("hidden", isSentOrDraft && !bothVisible);
  simplemail.domGet("to").setAttribute("hidden", !isSentOrDraft && !bothVisible);

  var search = simplemail.domGet("search").value;
  simplemail.domGet("folderId").hidden = !search;

  var oldCount = simplemail.messageTreeView.messages.length;
  simplemail.messageTreeView.messages = search ? simplemail.overlay.storage.getSearchMessages(search, simplemail.messageTreeView.getSort()) :
             folder ? simplemail.overlay.storage.getFolderMessages(folder.id, simplemail.messageTreeView.getSort()) : [];

  var selectionHandler = messageTree.onselect;
  messageTree.onselect = null;

  simplemail.messageTreeView.treebox.rowCountChanged(0, simplemail.messageTreeView.messages.length - oldCount);
  simplemail.messageTreeView.treebox.invalidate();
  simplemail.messageTreeView.selectFirstMessage();

  messageTree.onselect = selectionHandler;
  simplemail.messageTreeView.displayMessageHtml();
}

// userok: rewrited
simplemail.messageTreeView.onMessagesDeleted = function(deleted) {
  if (!deleted) 
    return;

  function isdeleted(message) {
    for (var i in deleted) {
      if (deleted[i].id == message.id) 
        return true;
    }
  }
  
  var selectionHandler = messageTree.onselect;
  messageTree.onselect = null;
  var index            = messageTree.currentIndex;
  
  simplemail.messageTreeView.treebox.beginUpdateBatch();
  for (var i = simplemail.messageTreeView.messages.length - 1; i >= 0; i--) {
    if (isdeleted(simplemail.messageTreeView.messages[i])) {
      simplemail.messageTreeView.treebox.rowCountChanged(i, -1);
      simplemail.messageTreeView.messages.splice(i, 1);
      if (i <= index) 
        index = index - 1;
    }
  }
  messageTree.onselect = selectionHandler;
  simplemail.messageTreeView.treebox.endUpdateBatch();
  
  if (messageTree.currentIndex == -1) {
    index = Math.min(index + 1, simplemail.messageTreeView.messages.length - 1);
    if (index != -1) 
      messageTree.view.selection.select(index);
    //userok: update twice? else 
    //userok: update twice?  simplemail.messageTreeView.displayMessageHtml();
  }
  simplemail.SimpleMailManageFolder.updateFolderCounts();
  simplemail.messageTreeView.displayMessageHtml();
}

// userok: rewrited, async
simplemail.messageTreeView.moveToFolder = function(folderId) {
  var deleted = [];
  var arrIds  = [];
  var toSpam  = (folderId == simplemail.SimpleMailManageFolder.getFolderId("SPAM"));
  simplemail.messageTreeView.forEachMessagesSelected(function(index) {
    var msg      = simplemail.messageTreeView.messages[index];
    msg.folderId = folderId;
    // Manage SPAMs
    if (toSpam) {
      msg.spam = true;
      processFilterSPAM(msg.from);
    } else if (msg.spam) {
      msg.spam = false;
      deleteRuleFilterSPAM(msg.from);
    }
    deleted.push(msg);
    arrIds.push(msg.id);
  });
  // userok: async
  simplemail.overlay.storage.moveMessagesToFolder(function(aMessagesIds, aFolderId, toSpam) {
    simplemail.messageTreeView.onMessagesDeleted(deleted);
  }, arrIds, folderId, toSpam);
}

simplemail.messageTreeView.getSort = function() {
  var column = simplemail.messageTreeView.getSortedColumn(); 
  if (!column) return;

  var direction = column.getAttribute("sortDirection");
  if (direction) {
    var sortColumns = {
      "image": "accountId",
      "from" : "fromm",
      "to"   : "too",
      "color": "noColor, color"
    };
    var id = sortColumns[column.id] || column.id;
    return id + " " + (direction == "descending" ? "DESC" : "ASC");
  }
}

// Partially downloaded simplemail.messageTreeView.messages

simplemail.messageTreeView.populateMessageMenu = function() {
  simplemail.domGet("message-menu-download").hidden = !simplemail.messageTreeView.isPartialSelected();
  return messageTree.view.selection.getRangeCount() > 0;
}

simplemail.messageTreeView.isPartialSelected = function() {
  simplemail.messageTreeView.isPartial = false;

  simplemail.messageTreeView.forEachMessagesSelected(function(index) {
    if (simplemail.messageTreeView.messages[index].partial) simplemail.messageTreeView.isPartial = true;
  });
  return simplemail.messageTreeView.isPartial;
}

simplemail.messageTreeView.downloadMessageBody = function() {
  simplemail.messageTreeView.partial = new Array();
  simplemail.messageTreeView.forEachMessagesSelected(function(index) {
    var message = simplemail.messageTreeView.messages[index];
    if (message.partial) simplemail.messageTreeView.partial[message.id] = message;
  });
  simplemail.overlay.getMailBody(simplemail.messageTreeView.partial);
}

// userok: rewrited
simplemail.messageTreeView.setMessageColor = function(color) {
  //console.error("SM.setMessageColor", color);//userok_log
  simplemail.messageTreeView.forEachMessagesSelected(function(index) {
    var message = simplemail.messageTreeView.messages[index];
    message.color = parseInt(color.replace('#', ''), 16);
    simplemail.overlay.storage.saveMessageState(message);
    simplemail.messageTreeView.treebox.invalidateRow(index);
  });
}

// userok
simplemail.messageTreeView.setMessageColorClick = function(event) {
  //console.error("SM.setMessageColorClick", event);//userok_log
  simplemail.domGet('message-menu').hidePopup();
  setTimeout(function(evt) {
    var clr = evt.target.color;
    if (clr)
      simplemail.messageTreeView.setMessageColor(clr);
  }, 15, event);
}

simplemail.messageTreeView.populateCharsetMenu = function(event) {
  var menupopup = event.target;
  simplemail.clearElement(menupopup);

  var charsets = simplemail.getDefaultCharsets();
  for (var i = 0; i < charsets.length; i++) {
    var menuitem = document.createElement("menuitem");
    menuitem.setAttribute("label", charsets[i]);
    menupopup.appendChild(menuitem);
  }
}

simplemail.messageTreeView.recodeMessage = function(charset) {
  simplemail.messageTreeView.forEachMessagesSelected(function(index) {
    var message = simplemail.overlay.storage.getMessage(simplemail.messageTreeView.messages[index].id);
    message.recode(charset);
    simplemail.overlay.storage.saveMessage(message, true);
    message.html = null; // Don't keep in memory
    simplemail.messageTreeView.messages[index] = message;
    simplemail.messageTreeView.treebox.invalidateRow(index);
    if (index == messageTree.currentIndex) 
      simplemail.messageTreeView.displayMessageHtml();
  });
}

// userok: rewrited
simplemail.messageTreeView.populateRefreshMenu = function() {
  var refresh = simplemail.domGet("refresh");
  return simplemail.overlay.populateMenu(refresh);
}

simplemail.messageTreeView.saveAsEmail = function(file, message) {
  var source = simplemail.SimpleMailFile.getAttachmentsDirectory(message.attachmentsDir);
  source.append(simplemail.SimpleMailFile.MESSAGE_SOURCE);
  source.copyTo(file.parent, file.leafName);
}

simplemail.messageTreeView.saveAsHtml = function(file, message) {
  message = simplemail.overlay.storage.getMessage(message.id);
  var html = '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN">' +
             '<meta http-equiv="Content-Type" content="text/html; charset=utf-8">' +
             '<style>' + simplemail.SimpleMailFile.readURL("chrome://simplemail/skin/message.css") + '</style>' +
             '<title>' + message.subject + '</title>' + message.toHtml();

  if (message.attachmentsCount) {
    var dir = file.clone();
    dir.leafName = dir.leafName.replace(/(\.[^.]*)?$/, "_files");
    if (!dir.exists()) dir.create(dir.DIRECTORY_TYPE, 0777);

    html = simplemail.SimpleMailText.replaceURLs(html,
      function(text, url) {
        if (url.match(/^simplemail:/i)) {
          var fileName = simplemail.SimpleMailFile.getFileName(url);

          var file = dir.clone();
          file.append(fileName);
          if (file.exists()) file.remove(false);
          file = simplemail.SimpleMailFile.getAttachmentsDirectory(message.attachmentsDir);
          file.append(fileName);
          file.copyTo(dir, fileName);

          return text.replace(url, dir.leafName + "/" + fileName);
        }
        return text;
      });
    html = simplemail.SimpleMailText.imageToLink(html);
  }
  simplemail.SimpleMailFile.writeFile(file, html, true);
}

simplemail.messageTreeView.clearSearch = function() {
  simplemail.domGet("search").value = "";
}

simplemail.messageTreeView.addToAddressBook = function() {
  simplemail.messageTreeView.forEachMessagesSelected(function(index) {
    var message = simplemail.messageTreeView.messages[index];
    // userok: async
    simplemail.overlay.storage.addAddressList(function(taddr) {
      simplemail.addressTreeView.treebox.rowCountChanged(simplemail.addressTreeView.rowCount - 1, 1);
    }, message.from);
  });
}

simplemail.messageTreeView.onRefreshClicked = function(event) {
  if (event.type == "dblclick") 
    simplemail.overlay.stop();
  else 
    simplemail.overlay.getMail();
}