if (typeof(simplemail) == "undefined")
  var simplemail = {a: "model.js"};

simplemail.COLORS = ["black", "red", "orange", "green", "blue", "violet"];

// userok: rewrited_class
simplemail.Address = function(id, name, email, group, favorite, postaladdress, phone) {
  this.id            = id;
  this.name          = name;
  this.email         = email;
  this.group         = group;
  this.favorite      = favorite;
  this.postaladdress = postaladdress;
  this.phone         = phone;
}

simplemail.AddressParse = function(text) {
  if (!text) 
    return;

  var name, email;
  text = text.replace(/[^'"\s<>]+@[^'"\s<>]+/, function($0) {
    email = $0;
    return "";
  });

  name = text.replace(/^['"\s<>]*|['"\s<>]*$/g, "");
  if (name || email)
    return new simplemail.Address(null, name, email, "", 0, "", "");
}

simplemail.AddressParseList = function(text) {
  var addresses = [];
  if (text) {
    var list = ("" + text).split(/[,;]/);
    for(var i in list) {
      var address = simplemail.AddressParse(list[i]);
      if (address) 
        addresses.push(address);
    }
  }
  return addresses;
}

simplemail.Address.prototype.getLogin = function() {
  return /(.*)@/.exec(this.email)[1];
}

simplemail.Address.prototype.getHost = function() {
  return /@(.*)/.exec(this.email)[1];
}

simplemail.Address.prototype.toString = function() {
  return this.name + (this.name && this.email ? " " : "") +
         (this.email ? "<" + this.email + ">" : "");
}

simplemail.Address.prototype.toHtml = function() {
  return this.name + (this.name && this.email ? " " : "") +
         (this.email ? "&lt;<a href='mailto:" + this.email + "'>" +
                       this.email + "</a>&gt;" : "");
}

// userok: rewrited_class
simplemail.SubAccount = function(parent, type, host, port, ssl, tls, auth, login, isImap) {
  this.parent = parent;
  this.type   = type;
  this.host   = host;
  this.port   = port;
  this.ssl    = ssl;
  this.tls    = tls;
  this.auth   = auth;
  this.login  = login;
  this.isImap = isImap;
}

simplemail.SubAccount.prototype.getPassword = function() {
  return simplemail.PasswordManager.getPassword(this.parent.id + this.type);
}

simplemail.SubAccount.prototype.setPassword = function(apassword) {
  return simplemail.PasswordManager.setPassword(this.parent.id + this.type, apassword);
}

// userok: rewrited_class
simplemail.Account = function(id, index, name, email, replyTo, refresh, deleteMode, color, subject, signature, isImap, pop3auth, pop3host, pop3port, pop3ssl, pop3login,
 smtpHost, smtpPort, smtpSsl, smtpLogin, smtpTls, smtpAuth) {
  this.id         = id;
  this.index      = index;
  this.name       = name;
  this.email      = email;
  this.replyTo    = replyTo;
  this.refresh    = refresh;
  this.deleteMode = deleteMode;
  this.color      = color;
  this.subject    = subject;
  this.signature  = signature;
  this.pop3       = new simplemail.SubAccount(this, "/pop3/", pop3host, pop3port, pop3ssl, 0, pop3auth, pop3login, isImap);
  this.smtp       = new simplemail.SubAccount(this, "/smtp/", smtpHost, smtpPort, smtpSsl, smtpTls, smtpAuth, smtpLogin, 0);
}

simplemail.Account.ASK        = "ask";
simplemail.Account.ALWAYS     = "always";
simplemail.Account.ON_RECEIVE = "onreceive";
simplemail.Account.NEVER      = "never";

simplemail.Account.prototype.toString = function() {
  return (this.name ? this.name + " " : "") + "<" + this.email + ">";
}

// userok: rewrited_class
simplemail.Message = function(id, accountId, uid, from, to, cc, bcc, subject, date, size, charset,
 read, deleted, attachmentsDir, attachmentsCount, html, folderId, partial, template, color, returnReceiptTo, replyTo, spam, replied, fwd) {
  this.id = id;
  this.accountId = accountId;
  this.uid = uid;
  this.from = from;
  this.to = to || "";
  this.cc = cc || "";
  this.bcc = bcc || "";
  this.subject = subject;
  this.date = date;
  this.size = size;
  this.charset = charset;
  this.read = read;
  this.deleted = deleted;
  this.attachmentsDir = attachmentsDir || simplemail.getUniqueId();
  this.attachmentsCount = attachmentsCount || 0;
  this.html = html;
  this.folderId = folderId;
  this.partial = partial;
  this.template = template;
  this.color = color;
  this.returnReceiptTo = returnReceiptTo;
  this.replyTo = replyTo;
  this.spam = spam;
  this.replied = replied;
  this.fwd = fwd;
    
  this.contentType;
  this.source;
}

simplemail.MessageFields = ["from", "to", "cc", "bcc", "subject", "charset", "html", "returnReceiptTo"];

simplemail.MessageParse = function(source, create) {
  return simplemail.SimpleMailMessageEncoder.decode(source, create);
}

simplemail.MessageParseMailto = function(mailto) {
  function getUrlParameter(name) {
    var regexp = new RegExp("[?&]" + name + "=([^&]*)", "i");
    var result = regexp.exec(mailto);
    return result ? decodeURIComponent(result[1]) : null;
  }
  var message = new simplemail.Message();
  message.to      = decodeURIComponent(/mailto:([^?&]*)/.exec(mailto)[1]);
  message.cc      = getUrlParameter("cc");
  message.bcc     = getUrlParameter("bcc");
  message.subject = getUrlParameter("subject");
  message.html    = getUrlParameter("body");
  return message;
}
 
simplemail.Message.prototype.getSize = function() {
  if (this.size == null)
    this.size = this.toString().length;

  return this.size;
}

simplemail.Message.prototype.toString = function() {
  if (!this.source) {
    this.source = simplemail.SimpleMailMessageEncoder.encode(this);
  }
  return this.source;
}

simplemail.Message.prototype.toHtml = function() {
  function mailList2html(value) {
    var addresses = simplemail.AddressParseList(value);
    var result = "";
    if (simplemail.SimpleMailPrefs.getBool("plaintextOnSend")) {
      for(var i in addresses) {
        result += (result ? ", " : "") + addresses[i].toString();
      }
      result = result.replace(/<[\/]{0,1}/ig,"(");
      result = result.replace(/>[\/]{0,1}/ig,")");
      return result;
    } else {
      for(var i in addresses) {
        result += (result ? ", " : "") + addresses[i].toHtml();
      }
      return result;
    }
  }

  if (simplemail.SimpleMailPrefs.getBool("plaintextOnSend") && this.from) {
    this.from = this.from.replace(/<[\/]{0,1}/ig,"(");
    this.from = this.from.replace(/>[\/]{0,1}/ig,")");
    var from = this.from
  } else {
    var from = this.from ? simplemail.AddressParse(this.from).toHtml() : "";
  }
  return "<span style='color: gray;'>" +
         "From: " + from + "<br>" +
         "To: " + mailList2html(this.to) + "<br>" + 
         (this.cc ? "Cc: " + mailList2html(this.cc) + "<br>" : "") +
         "Date: " + simplemail.SimpleMailDate.toString(this.date) + "<br>" +
         "Subject: " + this.subject + "<br>" +
         "</span><br>" + this.html;
}

simplemail.Message.prototype.toHtmlReceipt = function() {
  function mailList2html(value) {
    var addresses = simplemail.AddressParseList(value);
    var result = "";
    for (var i in addresses) {
      result += (result ? ", " : "") + addresses[i].toHtml();
    }
    return result;
  }
  var from = this.from ? simplemail.AddressParse(this.from).toHtml() : "";
  return "<span style='color: gray;'>" +
         "From: " + from + "<br>" +
         "To: " + mailList2html(this.to) + "<br>" + 
         (this.cc ? "Cc: " + mailList2html(this.cc) + "<br>" : "") +
         "Date: " + simplemail.SimpleMailDate.toString(this.date) + "<br>" +
         "Subject: " + this.subject + "<br>" +
         "</span><br>";
}

simplemail.Message.prototype.createReply = function(all) {
  var reply       = new simplemail.Message();
  reply.accountId = this.accountId;
  reply.charset   = this.charset;
  reply.to        = this.replyTo || this.from;
  if (all) {
    if (this.to) 
      reply.to += ", " + this.to;
    reply.cc  = this.cc;
    reply.bcc = this.bcc;
  }
  reply.subject = this.subject.replace(/^(Re: )?/i, "Re: ");
  // Remove attachments
  reply.html = simplemail.SimpleMailText.replaceURLs(this.html,
    function(text, url) {
      return simplemail.SimpleMailFile.isLocalURL(url)
         ? simplemail.Bundle.getString("attachments") + " : " + "[" + simplemail.SimpleMailFile.getFileName(url) + "]" + "<br>" : text;
    });

  if (simplemail.SimpleMailPrefs.getBool("plaintextOnSend")) {
    this.from = this.from.replace(/<[\/]{0,1}/ig,"(");
    this.from = this.from.replace(/>[\/]{0,1}/ig,")");
    reply.html = "<br><br><span style='color: gray;'>" +
                 simplemail.SimpleMailPrefs.getString("reply")
                        .replace(/{author}/gi, this.from)
                        .replace(/{date}/gi, simplemail.SimpleMailDate.toString(this.date))
                        .replace(/{date-gmt}/gi, simplemail.SimpleMailDate.toGMT(this.date))
                        .replace(/{date-locale}/gi, new Date(this.date).toLocaleString())
                        .replace(/{subject}/gi, this.subject) +
                 "</span><br>" +
                 "<span style='font-style: italic'>" + reply.html + "</span>"
    return reply;
  } else {
    reply.html = "<br><br><span style='color: gray;'>" +
                 simplemail.SimpleMailPrefs.getString("reply")
                        .replace(/{author}/gi, simplemail.AddressParse(this.from).toHtml())
                        .replace(/{date}/gi, simplemail.SimpleMailDate.toString(this.date))
                        .replace(/{date-gmt}/gi, simplemail.SimpleMailDate.toGMT(this.date))
                        .replace(/{date-locale}/gi, new Date(this.date).toLocaleString())
                        .replace(/{subject}/gi, this.subject) +
                 "</span><br>" +
                 "<blockquote style='border-left: 1px solid rgb(204, 204, 204);" +
                                    "margin: 0 0 0 0.8ex;" +
                                    "padding: 1ex 0 0 1ex;'>" + reply.html + "</blockquote>";
    return reply;      
  }                                
}

simplemail.Message.prototype.createForward = function() {
  var forward = new simplemail.Message();
  forward.accountId = this.accountId;
  forward.charset = this.charset;
  forward.subject = this.subject.replace(/^(Fwd?: )?/i, "Fwd: ");
  if (simplemail.SimpleMailPrefs.getBool("plaintextOnSend")) {
    forward.html = "<br><br> -- Message -- <br><br>" + this.toHtml();
    return forward;
  } else {
    forward.html = "<br><br><hr>" + this.toHtml();
    return forward;
  }
}

simplemail.Message.prototype.createReturnReceipt = function(recipient, message) {
  var source = simplemail.SimpleMailFile.getAttachmentsDirectory(message.attachmentsDir);
  source.append(simplemail.SimpleMailFile.MESSAGE_SOURCE);
  source = simplemail.SimpleMailFile.readFile(source);
  var originalMessageID = source.match(/^Message-ID:[^<]*<(.*?)>/gim)
  originalMessageID = originalMessageID.toString().match(/<.*>/);
      
  var receipt = new simplemail.Message();
  receipt.accountId = this.accountId;
  receipt.from = recipient;
  receipt.to = this.returnReceiptTo;
  receipt.charset = "utf-8";
  receipt.subject = "Rcpt: " + this.subject;

  var html = simplemail.Bundle.getString("receipt") + "<br>" + simplemail.Bundle.getString("date") +" : " + simplemail.SimpleMailDate.toString(new Date().getTime()) + " - " + simplemail.Bundle.getString("messageReceived")+" :<br><hr>";

  // Create MDNFile.txt
  var file = simplemail.SimpleMailFile.getAttachmentsDirectory(receipt.attachmentsDir);
  file.append("MDNFile.txt");
  var out = simplemail.SimpleMailFile.createOutputStream(file);
  out.write("Reporting-UA: SimpleMail" + "\r\n");
  out.write("Final-Recipient: rfc822;" + receipt.from + "\r\n");
  out.write("Original-Message-ID: " + originalMessageID + "\r\n");
  if (simplemail.SimpleMailPrefs.getString("sendreceiptresp") == "asksend") {
    out.write("Disposition: manual-action/MDN-sent-manually; displayed" + "\r\n");
  } else {
    out.write("Disposition: automatic-action/MDN-sent-automatically; displayed" + "\r\n");
  }
  out.close();
        
  // Attach MDNFile.txt
  var url = simplemail.SimpleMailFile.getFileURL(file);
  var filename = simplemail.SimpleMailFile.getFileName(url);
  var background = simplemail.SimpleMailMessageEncoder.getIconForAttachment(filename);
  html += "<a class='attachment' style='background: url(" + background + ")center left no-repeat;' href='" + url + "'>" + filename + "</a>";
  receipt.html = html + this.toHtmlReceipt() + "<hr>";
  return receipt;
}

simplemail.Message.prototype.equals = function(message) {
  if (message == null) 
    return true;
  
  for (var i in simplemail.MessageFields) {
    var field = simplemail.MessageFields[i];
    if (this[field] != message[field]) 
      return false;
  }
  return true;
}

simplemail.Message.prototype.copyTo = function(message) {
  for (var i in simplemail.MessageFields) {
    var field = simplemail.MessageFields[i];
    message[field] = this[field];
  }
}

simplemail.Message.prototype.recode = function(charset) {
  try {
    var message = new simplemail.Message();
    for (var i in simplemail.MessageFields) {
      var field = simplemail.MessageFields[i];
      message[field] = simplemail.SimpleMailText.fromUnicode(this[field], this.charset);
      message[field] = simplemail.SimpleMailText.toUnicode(message[field], charset);
    }
    message.charset = charset;
    // If no exceptions, update original message
    message.copyTo(this);
  } catch(e) {
    
  }
}

// userok: rewrited_class
simplemail.Folder = function(id, parentId, name, opened) {
  this.id       = id;
  this.parentId = parentId;
  this.name     = name;
  this.open     = opened;
}

simplemail.Folder.prototype.getName = function() {
  return simplemail.Bundle.getString(this.name) || this.name;
}

simplemail.Folder.DEFAULT_NAMES = ["inbox", "sent", "drafts", "trash"];
simplemail.Folder.INBOX  = 1;
simplemail.Folder.SENT   = 2;
simplemail.Folder.DRAFTS = 3;
simplemail.Folder.TRASH  = 4;

// userok: rewrited_class
simplemail.Rule = function(id, filterId, property, not, condition, param) {
  this.id        = id;
  this.filterId  = filterId;
  this.property  = property;
  this.not       = not;
  this.condition = condition;
  this.param     = param;
}

simplemail.RuleGetProperties = function() {   
 return ["accountId", "from", "to", "subject", "html", "size", "source", "read"];
}

simplemail.RuleGetConditions = function(aProperty) {
  switch (aProperty) {
    case "read":
      return [""];
    case "accountId":
      return ["equals"];
    case "from":
      return ["contains", "inAddressBook"];
    case "size":
      return ["greaterThan"];
    default:
      return ["contains"];
  }
}

simplemail.Rule.prototype.matchAll = function(string, patterns) {
  string = String.toLowerCase(string);
  for(var i = 0; i < patterns.length; i++) {
    var pattern = simplemail.SimpleMailText.trim(patterns[i]).toLowerCase();
    if (string.indexOf(pattern) == -1) 
      return false;
  }
  return true;
}

simplemail.Rule.prototype.test = function(message, storage, mail) {
  var value = message[this.property];
  var result;
  //console.error("SM.model_ruletest", result, value, message, mail, this);//userok_log
  switch (this.condition) {
    case "": // userok: toggle for read\unread
      result = (value);
      break;
    case "equals":
      result = (value == this.param);
      break;
    case "contains":
      // "," separator means "or", "+" means "and"
      var any = this.param.split(/,/);
      for (var i = 0; !result && i < any.length; i++) {
        var all = any[i].split(/\+/);
        result |= this.matchAll(value, all);
      }
      break;
    case "inAddressBook":
      var address = simplemail.AddressParse(value);
      result      = address && storage.getAddressByEmail(address.email);
      break;
    case "greaterThan":
      result = (parseInt(value) > parseInt(this.param));
      break;
    default:
      simplemail.error("Unknown condition: " + this.condition);
      break;
  }

  if (this.not & result == 1) {
    return 0;
  } else if (this.not & result == 0) {
    return 1
  } else 
    return result;
}

// userok: rewrited_class
simplemail.Filter = function(id, index, name, type, rules, actions) {
  this.id       = id;
  this.index    = index;
  this.name     = name;
  this.type     = type || simplemail.Filter.INBOX;
  this.rules    = rules;
  this.actions  = actions;
}

simplemail.Filter.INBOX = 1;
simplemail.Filter.SENT  = 2;

simplemail.Filter.prototype.getName = function() {
  return simplemail.Bundle.getString(this.name) || this.name;
}

simplemail.Filter.prototype.execute = function(message, storage, mail) {
  //console.error("SM.SimpleMailFilter_execute", this);//userok_log
  var len = this.rules.length;
  for (var i = 0; i < len; i++) {
    if (this.rules[i].test(message, storage, mail)) {
      for (var j = 0; message && j < this.actions.length; j++)
        message = this.actions[j].execute(message, storage, mail);
    }
  }
  return message;
}

// userok: rewrited_class
simplemail.FilterManager = function(storage, mail) {
  this.storage = storage;
  this.mail    = mail;
}

simplemail.FilterManager.prototype.execute = function(message, type) {
  var tfilters = this.storage.getFilters();
  for (var i = 0; message && i < tfilters.length; i++) {
    var filter = tfilters[i];
    //console.error("SM.SimpleMailFilter_execute", typeof(filter), filter, filter.execute);//userok_log
    // Check if a rule is present
    if (type == filter.type && filter.rules[0]) {
      // Don't apply SPAM filter if SPAM not managed
      if (simplemail.SimpleMailPrefs.getBool("nospam") && filter.name !== "SPAM")
        message = filter.execute(message, this.storage, this.mail)
      else if (!simplemail.SimpleMailPrefs.getBool("nospam")) {
        var filterSPAM  = filter;
        var messageFrom = message.from;
        for (var j in filterSPAM.rules) {
          if (filterSPAM.rules[j].param == messageFrom) {
            message.spam = true;
            if (simplemail.SimpleMailPrefs.getBool("markspamasread"))
              message.read = true;
          }
          this.storage.saveMessageState(message);
        }
        message = filter.execute(message, this.storage, this.mail);
      }
    }
  } 
  return message;          
}

// userok: rewrited_class
simplemail.Action = function(id, filterId, name, param) {
  this.id       = id;
  this.filterId = filterId;
  this.name     = name;
  this.param    = param;
}

simplemail.ActionGetNames = function() {
  return [ "sound", "color", "folder", "delete", "download", "charset", "forward" ];
}

simplemail.Action.prototype.execute = function(message, storage, mail) {
  //console.error("SM.SimpleMailAction_execute", this, simplemail);//userok_log
  switch (this.name) {
    case "sound":
      simplemail.SimpleMailSound.play(this.param);
      break;
    case "color":
      message.color = parseInt(this.param.replace('#', ''), 16);
      break;
    case "folder":
      if (storage.getFolder(this.param))
        message.folderId = this.param;
      break;
    case "delete":
      mail.delete(message);
      return;
    case "download":
      if (message.partial) {
        mail.getBody(message);
        return;
      }
      break;
    case "charset":
      message.recode(this.param);
      break;
    case "forward":
      simplemail.SimpleMailPrefs.setString('forwardParam', this.param);
      break;          
    case "popup":
    case "counter":
      break; // Obsolete
    default:
      simplemail.error("Unknown action: " + this.name);
      break;
  }
  return message;
}
