function editactions_initialize() {
  var action     = window.arguments[0].action;
  document.title = action.id ? simplemail.Bundle.getString("editAction")
                             : simplemail.Bundle.getString("newAction");
  editactions_populateName();
  editactions_populateFolder();
  editactions_populateCharset();

  simplemail.selectItem("name", action.name);
  switch (action.name) {
    case "folder":
      simplemail.selectItem("folder", action.param);
      break;
    case "sound":
      simplemail.domGet("sound-path").value = action.param;
      break;
    case "color": // userok: rewrited
      simplemail.domGet("color").color = action.param;
      break;
    case "charset":
      simplemail.selectItem("charset", action.param);
      break;
    case "forward":
      simplemail.domGet("forward-mail").value = action.param;
      break;
  }
  editactions_onSelectName();
}

function editactions_onAccept() {
  var action  = window.arguments[0].action;
  action.name = simplemail.domGet("name").selectedItem.value;
  switch (action.name) {
    case "folder":
      action.param = simplemail.domGet("folder").selectedItem.value;
      break;
    case "sound":
      action.param = simplemail.domGet("sound-path").value;
      break;
    case "color": // userok: rewrited
      action.param = simplemail.domGet("color").color;
      break;
    case "charset":
      action.param = simplemail.domGet("charset").selectedItem.label;
      break;
    case "forward":
      action.param = simplemail.domGet("forward-mail").value;
      break;
    default:
      action.param = null;
      break;
  }
}

function editactions_onCancel() {
  window.arguments[0].action = null;
}

function editactions_populateName() {
  var names = simplemail.ActionGetNames();
  var popup = simplemail.domGet("name").menupopup;
  for (var i in names) {
    var menuitem = document.createElement("menuitem");
    menuitem.setAttribute("label", simplemail.Bundle.getString(names[i]));
    menuitem.setAttribute("value", names[i]);
    popup.appendChild(menuitem);
  }
}

function editactions_populateFolder() {
  simplemail.overlay = simplemail.getBrowserWindow().simplemail.overlay;
  var tfolders = simplemail.overlay.storage.getFolders();

  function forEachFolder(folderId, onFolder, level) {
    if (!level) level = 0;
    var children = tfolders[folderId];
    for (var i in children) {
      onFolder(children[i], level);
      forEachFolder(children[i].id, onFolder, level + 1);
    }
  }
  
  var popup = simplemail.domGet("folder").menupopup;
  forEachFolder(0, function(folder, level) {
    var indent = "";
    for(var i = 0; i < level; i++) indent += "  ";
    var menuitem = document.createElement("menuitem");
    menuitem.setAttribute("label", indent + folder.getName());
    menuitem.setAttribute("value", folder.id);
    popup.appendChild(menuitem);
  });
  
  popup.parentNode.selectedIndex = 0;
}

function editactions_populateCharset() {
  var popup    = simplemail.domGet("charset").menupopup;
  var charsets = simplemail.getDefaultCharsets();
  for(var i = 0; i < charsets.length; i++) {
    var menuitem = document.createElement("menuitem");
    menuitem.setAttribute("label", charsets[i]);
    popup.appendChild(menuitem);
  }
  popup.parentNode.selectedIndex = 0;
}

function editactions_onSelectName() {
  var name  = simplemail.domGet("name").selectedItem.value;
  var names = simplemail.ActionGetNames();
  var param;

  for (var i in names) {
    var element = simplemail.domGet(names[i]);
    if (element) {
      element.hidden = (names[i] != name);
      if (names[i] == name) 
        param = element;
    }
  }
  simplemail.domGet("param-label").hidden = !param || param.nodeName == "label";
}

function editactions_browseSound() {
  var file = simplemail.SimpleMailFile.chooseFile("open", ["*.wav"]);
  if (file)
    simplemail.domGet("sound-path").value = file.path;
}

function editactions_clearSound() {
  simplemail.domGet("sound-path").value = null;
}

function editrules_initialize() {
  simplemail.overlay = simplemail.getBrowserWindow().simplemail.overlay;
  var rule = window.arguments[0].rule;
  
  document.title = rule.id ? simplemail.Bundle.getString("editRule") : simplemail.Bundle.getString("newRule");
  editrules_populateProperty();
  simplemail.selectItem("property", rule.property);
  simplemail.domGet("not").checked = rule.not;
  editrules_populateCondition();
  simplemail.selectItem("condition", rule.condition);
  editrules_populateSize();
  editrules_populateAccountId();
  editrules_setParam(rule.param);
}

function editrules_onAccept() {
  var rule       = window.arguments[0].rule;
  rule.property  = simplemail.domGet("property").selectedItem.value;
  rule.not       = simplemail.domGet("not").checked;
  rule.condition = simplemail.domGet("condition").selectedItem.value;
  rule.param     = editrules_getParam();
}

function editrules_onCancel() {
  window.arguments[0].rule = null;
}

function editrules_populateMenuList(menuId, values) {
  var popup = simplemail.domGet(menuId).menupopup;
  simplemail.clearElement(popup);

  for (var i = 0; i < values.length; i++) {
    var menuitem = document.createElement("menuitem");
    menuitem.setAttribute("label", simplemail.Bundle.getString(values[i]));
    menuitem.setAttribute("value", values[i]);
    popup.appendChild(menuitem);
  }
  popup.parentNode.selectedIndex = 0;
}

function editrules_populateProperty() {
  editrules_populateMenuList("property", simplemail.RuleGetProperties());
}

function editrules_populateCondition() {
  var property = simplemail.domGet("property").value;
  editrules_populateMenuList("condition", simplemail.RuleGetConditions(property));
}

function editrules_populateSize() {
  editrules_populateMenuList("size", ["bytes", "kb", "mb"]);
}

function editrules_setParam(value) {
  var property = simplemail.domGet("property").selectedItem.value;
  switch (property) {
    case "size":
      if (value > 1000000) {
        value = value / 1000000;
        simplemail.selectItem("size", "mb");
      } else if (value > 1000) {
        value = value / 1000;
        simplemail.selectItem("size", "kb");
      }
      break;
    case "accountId":
      simplemail.selectItem("accountId", value);
      break;
  }
  simplemail.domGet("param").value = value;
  editrules_showParam();
}

function editrules_getParam() {
  var condition = simplemail.domGet("condition").selectedItem.value;
  if (condition == "inAddressBook") 
    return;

  var value    = simplemail.domGet("param").value;
  var property = simplemail.domGet("property").selectedItem.value;
  switch(property) {
    case "size":
      var size = simplemail.domGet("size").selectedItem.value;
      switch (size) {
        case "mb": value = value * 1000000; break;
        case "kb": value = value * 1000;    break;
      }
      value = value || 0;
      break;
    case "accountId":
      value = simplemail.domGet("accountId").selectedItem.value;
      break;
  }
  return value;
}

// userok: rewrited
function editrules_showParam() {
  var property  = simplemail.domGet("property").selectedItem.value;
  var condition = simplemail.domGet("condition").selectedItem.value;
  simplemail.domGet("param").hidden       = condition == "inAddressBook" || property == "accountId" || property == "read";
  simplemail.domGet("param-label").hidden = condition == "inAddressBook" || property == "read";
  simplemail.domGet("size").hidden        = property != "size" || property == "read";
  simplemail.domGet("accountId").hidden   = property != "accountId" || property == "read";
  simplemail.domGet("condition").hidden   = simplemail.domGet("condition-label").hidden = (property == "read");
}

function editrules_showTooltip(element) {
  var condition = simplemail.domGet("condition").selectedItem.value;
  if (condition == "contains")
    simplemail.domGet("tooltip").showPopup(element, -1, -1, "tooltip", null, "topright");
}

function editrules_hideTooltip() {
  simplemail.domGet("tooltip").hidePopup();
}

function editrules_populateAccountId() {
  var popup     = simplemail.domGet("accountId").menupopup;
  var taccounts = simplemail.overlay.storage.getAccounts();

  for (var i = 0; i < taccounts.length; i++) {
    var menuitem = document.createElement("menuitem");
    menuitem.setAttribute("label", taccounts[i]);
    menuitem.setAttribute("value", taccounts[i].id);
    popup.appendChild(menuitem);
  }
  popup.parentNode.selectedIndex = 0;
}

function filter_initialize() {
  simplemail.overlay = simplemail.getBrowserWindow().simplemail.overlay;
  var filter         = window.arguments[0].filter;
  
  filter_populateTypes();
  simplemail.selectItem("type", filter.type);

  var name = filter.getName();
  document.title = filter.id ? simplemail.Bundle.getString("editFilter", [name])
                             : simplemail.Bundle.getString("newFilter");
  simplemail.domGet("name").value = name;

  initRules(filter.rules);
  initActions(filter.actions);
}

function filter_onAccept() {
  var filter  = window.arguments[0].filter;
  filter.name = simplemail.domGet("name").value;
  filter.type = simplemail.domGet("type").selectedItem.value;
}

function filter_onCancel() {
  window.arguments[0].filter = null;
}

function filter_populateTypes() {
  var menupopup = simplemail.domGet("type").menupopup;
  var menuitem  = document.createElement("menuitem");
  menuitem.setAttribute("label", simplemail.Bundle.getString("inbox"));
  menuitem.setAttribute("value", simplemail.Filter.INBOX);
  menupopup.appendChild(menuitem);
  var menuitem = document.createElement("menuitem");
  menuitem.setAttribute("label", simplemail.Bundle.getString("sent"));
  menuitem.setAttribute("value", simplemail.Filter.SENT);
  menupopup.appendChild(menuitem);
}